from torch import nn
from cmvc import *





class Net(nn.Module):
  def __init__(self):
    
    
    super().__init__()
    self.ue = UttrEncoder()
    self.ud = UttrDecoder()
    self.fe = FaceEncoder()
    self.fd = FaceDecoder()
    self.ve = VoiceEncoder()

  def forward(self, x, y):
    z = self.ue(x)
    c = self.fe(y)
    x_hat = self.ud(z, c)
    print(x_hat.size())
    c_hat = self.ve(x_hat)
    print(c_hat.size())
    y_hat = self.fd(c_hat)

    return y_hat
  
  def loss(self, x, y):
    """
    reconstruction + KL divergence
    """
    pass


from torch import nn
from cmvc.scripts.layer import *





from math import ceil

def p(kernel_size, stride):
    if kernel_size == stride:
        return 0
    
    else:
        return ceil((kernel_size-1)/stride)


    
class UttrEncoder(nn.Module):
  def __init__(self):

    super().__init__()
    self.uttr_enc_d1 = CBGLayer(in_channels=1,
                                out_channels=16,
                                kernel_size=(3, 9),
                                stride = (1,1),
                                padding=(1,4))
    
    self.uttr_enc_d2 = CBGLayer(in_channels=16,
                                out_channels=32,
                                kernel_size=(4, 8),
                                stride = (2,2),
                                padding=(1,3))    

    self.uttr_enc_d3 = CBGLayer(in_channels=32,
                                out_channels=32,
                                kernel_size=(4, 8),
                                stride = (2,2),
                                padding=(1,3))    
    
    self.uttr_enc_d4 = nn.Conv2d(in_channels=32,
                             out_channels=16,
                             kernel_size=(9, 5),
                             stride = (9,1),
                             padding=(0,2))

  def uttr_encoder(self, x):
    """
    音声のencoder
    """

    x = self.uttr_enc_d1(x)
    x = self.uttr_enc_d2(x)
    x = self.uttr_enc_d3(x)
    x = self.uttr_enc_d4(x)

    mean, log_var = torch.split(x, 8, dim=1) # 半分
     
    return mean, log_var

  def uttr_sample_z(self, mean, log_var):
    """
    潜在変数出すやつ
    """
    epsilon = torch.randn(mean.shape).to(device)
    return mean + torch.exp(log_var) * epsilon

  def forward(self, x):
    mean, log_var = self.uttr_encoder(x)
    z = self.uttr_sample_z(mean, log_var)
    return z



class FaceEncoder(nn.Module):
  def __init__(self):

    super().__init__()
    self.face_enc_d1 = nn.Sequential(nn.Conv2d(in_channels=3,
                                               out_channels=32,
                                               kernel_size=(6, 6),
                                               stride = (2,2),
                                               padding=(2,2)),
                                     nn.LeakyReLU())

    self.face_enc_d2 = CBLLayer(in_channels=32,
                                out_channels=64,
                                kernel_size=(6, 6),
                                stride = (2,2),
                                padding=(2,2))
    
    self.face_enc_d3 = CBLLayer(in_channels=64,
                                out_channels=128,
                                kernel_size=(4, 4),
                                stride = (2,2),
                                padding=(1,1))
    
    self.face_enc_d4 = CBLLayer(in_channels=128,
                                out_channels=128,
                                kernel_size=(4, 4),
                                stride = (2,2),
                                padding=(1,1)
                                )
    
    self.face_enc_d5 = CBLLayer(in_channels=128,
                                out_channels=256,
                                kernel_size=(4, 4),
                                stride = (2,2),
                                padding=(1,1))
    
    self.face_enc_d6 = FlattenLayer() #flattenまでに(n, c, 1, 1)になってる前提

    self.face_enc_d7 = nn.Sequential(nn.Linear(256,256),
                                     nn.LeakyReLU())
    
    self.face_enc_d8 = nn.Sequential(nn.Linear(256,16),
                                     nn.LeakyReLU())

  def face_encoder(self, y):
    """
    顔面のencoder
    """

    y = self.face_enc_d1(y)
    y = self.face_enc_d2(y)
    y = self.face_enc_d3(y)
    y = self.face_enc_d4(y)
    y = self.face_enc_d5(y)
    y = self.face_enc_d6(y)
    y = self.face_enc_d7(y)
    y = self.face_enc_d8(y)

    
    
    mean, log_var = torch.split(y, 8, dim=1) # 半分
     
    return mean, log_var

  def face_sample_z(self, mean, log_var):
    """
    顔面の潜在変数出すやつ
    """
    epsilon = torch.randn(mean.shape).to(device)
    return mean + torch.exp(log_var) * epsilon

  def forward(self, y):
    mean, log_var = self.face_encoder(y)
    z = self.face_sample_z(mean, log_var)
    z = z.unsqueeze(-1).unsqueeze(-1)
    return z



class VoiceEncoder(nn.Module):
  def __init__(self):

    super().__init__()
    self.voice_enc_d1 = CBGLayer(in_channels=1,
                                 out_channels=32,
                                 kernel_size=(3,9),
                                 stride=(1,1),
                                 padding=(1,4))
    
    self.voice_enc_d2 = CBGLayer(in_channels=32,
                                 out_channels=64,
                                 kernel_size=(4,8),
                                 stride=(2,2),
                                 padding=(1,3))
    
    self.voice_enc_d3 = CBGLayer(in_channels=64,
                                 out_channels=128,
                                 kernel_size=(4,8),
                                 stride=(2,2),
                                 padding=(1,3))
    
    self.voice_enc_d4 = CBGLayer(in_channels=128,
                                 out_channels=128,
                                 kernel_size=(4,8),
                                 stride=(2,2),
                                 padding=(1,3))
    
    self.voice_enc_d5 = CBGLayer(in_channels=128,
                                 out_channels=128,
                                 kernel_size=(4,5),
                                 stride=(4,1),
                                 padding=(0,2))
    
    self.voice_enc_d6 = CBGLayer(in_channels=128,
                                 out_channels=64,
                                 kernel_size=(1,5),
                                 stride=(1,1),
                                 padding=(0,2))
    
    self.voice_enc_d7 = nn.Conv2d(in_channels=64,
                                  out_channels=16,
                                  kernel_size=(1,5),
                                  stride=(1,1),
                                 padding=(0,2))
                                 

  def voice_encoder(self, x):
    """
    顔面持ってくるencoder
    """
    x = self.voice_enc_d1(x)
    x = self.voice_enc_d2(x)
    x = self.voice_enc_d3(x)
    x = self.voice_enc_d4(x)
    x = self.voice_enc_d5(x)
    x = self.voice_enc_d6(x)
    x = self.voice_enc_d7(x)
        
    """
    第4層のConv2d出力のchannelの半分でmean半分でlog_varを予測している？
    """
    mean, log_var = torch.split(x, 8, dim=1) # 半分
     
    return mean, log_var

  def voice_sample_z(self, mean, log_var):
    """
    音声の潜在変数出すやつ
    """
    epsilon = torch.randn(mean.shape).to(device)
    return mean + torch.exp(log_var) * epsilon

  def forward(self, x):
    mean, log_var = self.voice_encoder(x)
    z = self.voice_sample_z(mean, log_var)

    z = z.squeeze(-1).squeeze(-1)
    return z



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cmvc', 'cmvc.scripts']

package_data = \
{'': ['*']}

install_requires = \
['torch>=1.8.1,<2.0.0']

setup_kwargs = {
    'name': 'cmvc',
    'version': '0.0.1',
    'description': 'This is a voice conversion model implemented in PyTorch.',
    'long_description': '# CMVC\n論文のモデルを実装しています。\n現在forwardが通ることを確認しています。\n\nhttps://arxiv.org/abs/1904.04540\n',
    'author': 'kjun1',
    'author_email': 'p.k.maejima1211@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kjun1/CMVC',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from torch import nn 





class CBGLayer(nn.Module):
  """
  Conv+Bn+GLU
  """
  def __init__(self, in_channels, out_channels, kernel_size, stride,padding=0):
    super().__init__()
    self.conv1 = nn.Conv2d(in_channels=in_channels,
                           out_channels=out_channels,
                           kernel_size=kernel_size,
                           stride = stride,
                           padding=padding)
    self.conv2 = nn.Conv2d(in_channels=in_channels,
                           out_channels=out_channels,
                           kernel_size=kernel_size,
                           stride = stride,
                           padding=padding)

    self.bn1 = nn.BatchNorm2d(out_channels)
    self.bn2 = nn.BatchNorm2d(out_channels)


  def forward(self, x):
    x1 = self.bn1(self.conv1(x))
    x2 = self.bn2(self.conv2(x))

    x = torch.cat((x1,x2),1)
    x = nn.functional.glu(x,1)
    
    return x



class CBLLayer(nn.Module):
  """
  Conv+Bn+LReLU
  """
  def __init__(self, in_channels, out_channels, kernel_size, stride,padding=0):
    super().__init__()
    self.conv = nn.Conv2d(in_channels=in_channels,
                          out_channels=out_channels,
                          kernel_size=kernel_size,
                          stride = stride,
                          padding=padding)

    self.bn = nn.BatchNorm2d(out_channels)

    self.lrelu = nn.LeakyReLU()


  def forward(self, x):
    x = self.conv(x)
    x = self.bn(x)
    x = self.lrelu(x)

    return x



class DBGLayer(nn.Module):
  """
  Deconv + Bn + GLU
  """
  def __init__(self, in_channels, out_channels, kernel_size, stride,padding=0):
    super().__init__()
    self.deconv1 = nn.ConvTranspose2d(in_channels=in_channels,
                                      out_channels=out_channels,
                                      kernel_size=kernel_size,
                                      stride=stride,
                                      padding=padding)
    self.deconv2 = nn.ConvTranspose2d(in_channels=in_channels,
                                      out_channels=out_channels,
                                      kernel_size=kernel_size,
                                      stride=stride,
                                      padding=padding)
    
    self.bn1 = nn.BatchNorm2d(out_channels)
    self.bn2 = nn.BatchNorm2d(out_channels)



  def forward(self, x):
    x1 = self.bn1(self.deconv1(x))
    x2 = self.bn2(self.deconv2(x))

    x = torch.cat((x1,x2),1)
    x = nn.functional.glu(x,1)
    
    return x



class DBSLayer(nn.Module):
  """
  Deconv + Bn + SoftPlus
  """
  def __init__(self, in_channels, out_channels, kernel_size, stride, padding=0):
    super().__init__()
    self.deconv = nn.ConvTranspose2d(in_channels=in_channels,
                                     out_channels=out_channels,
                                     kernel_size=kernel_size,
                                     stride=stride,
                                     padding=padding)
    
    self.bn = nn.BatchNorm2d(out_channels)

    self.softplus = nn.Softplus()


  def forward(self, x):
    x = self.deconv(x)
    x = self.bn(x)
    x = self.softplus(x)

    return x



class FlattenLayer(nn.Module):
  """
  (N, C, H, W)を(N, C*H*W)にする
  """
  def forward(self, x):
    sizes = x.size()
    return x.view(sizes[0],  -1)

class ReshapeLayer(nn.Module):
  """
  (N, C*H*W)を(N, C, H, W)にする
  """
  def forward(self, x, out_channel):
    sizes = x.size()
    h = int((sizes[1]/out_channel)**0.5)
    return x.view(sizes[0],  out_channel, h, h)


from torch import nn
from cmvc.scripts.layer import *





class UttrDecoder(nn.Module):
  def __init__(self):

    super().__init__()

    self.uttr_dec_d1 = DBGLayer(in_channels=8,
                                out_channels=16,
                                kernel_size=(9,5),
                                stride=(9,1),
                                padding=(0,2))
  
    self.uttr_dec_d2 = DBGLayer(in_channels=16,
                                out_channels=16,
                                kernel_size=(4,8),
                                stride=(2,2),
                                padding=(1,3))
  
    self.uttr_dec_d3 = DBGLayer(in_channels=16,
                                out_channels=8,
                                kernel_size=(4,8),
                                stride=(2,2),
                                padding=(1,3))
  
    self.uttr_dec_d4 = nn.ConvTranspose2d(in_channels=8,
                                          out_channels=2,
                                          kernel_size=(3,9),
                                          stride=(1,1),
                                          padding=(1,4))
  
  def uttr_decoder(self, z, c):
    #print(z.size())
    
    x,_ = torch.broadcast_tensors(z, c)
    x = self.uttr_dec_d1(x)
    #print(x.size())
    
    x,_ = torch.broadcast_tensors(x, torch.cat((c, c),1))
    x = self.uttr_dec_d2(x)
    #print(x.size())

    x,_ = torch.broadcast_tensors(x, torch.cat((c, c),1))
    x = self.uttr_dec_d3(x)
    #print(x.size())

    x,_ = torch.broadcast_tensors(x, c)
    x = self.uttr_dec_d4(x)
    #print(x.size())

    mean, log_var = torch.split(x, 1, dim=1) # 半分
     
    return mean, log_var
  

  def uttr_sample_z(self, mean, log_var):

    epsilon = torch.randn(mean.shape).to(device)
    return mean + torch.exp(log_var) * epsilon
  

  def forward(self, z, c):
    mean, log_var = self.uttr_decoder(z, c)
    z = self.uttr_sample_z(mean, log_var)
    return z



class FaceDecoder(nn.Module):
  def __init__(self):

    super().__init__()
    self.face_dec_d1 = nn.Sequential(nn.Linear(8,128),
                                     nn.Softplus())
    
    self.face_dec_d2 = nn.Sequential(nn.Linear(128,2048),
                                     nn.Softplus())
    
    self.face_dec_d3 = ReshapeLayer()

    self.face_dec_d4 = DBSLayer(in_channels=128,
                                out_channels=128,
                                kernel_size=(3,3),
                                stride=(2,2),
                                padding=(2,2))

    self.face_dec_d5 = DBSLayer(in_channels=128,
                                out_channels=128,
                                kernel_size=(6,6),
                                stride=(2,2),
                                padding=(2,2))
    
    self.face_dec_d6 = DBSLayer(in_channels=128,
                                out_channels=64,
                                kernel_size=(6,6),
                                stride=(2,2),
                                padding=(2,2))
    
    self.face_dec_d7 = DBSLayer(in_channels=64,
                                out_channels=32,
                                kernel_size=(6,6),
                                stride=(2,2),
                                padding=(2,2))
    
    self.face_dec_d8 = nn.Conv2d(in_channels=32,
                                 out_channels=6,
                                 kernel_size=(5,5),
                                 stride=(1,1))
    
    

  def face_decoder(self, c):

    y = self.face_dec_d1(c)
    y = self.face_dec_d2(y)
    y = self.face_dec_d3(y, 128)
    y = self.face_dec_d4(y)
    y = self.face_dec_d5(y)
    y = self.face_dec_d6(y)
    y = self.face_dec_d7(y)
    y = self.face_dec_d8(y)

    mean, log_var = torch.split(y, 3, dim=1) # 半分
     
    return mean, log_var

  def face_sample_z(self, mean, log_var):
    """
    顔面の潜在変数出すやつ
    """
    epsilon = torch.randn(mean.shape).to(device)
    return mean + torch.exp(log_var) * epsilon

  def forward(self, y):
    mean, log_var = self.face_decoder(y)
    z = self.face_sample_z(mean, log_var)
    return z
